@@ -1,6 +1,6 @@
 # et:ts=4
 # portchecksum.tcl
-# $Id: portchecksum.tcl 51521 2009-05-27 08:48:05Z jmr@macports.org $
+# $Id: portchecksum.tcl 70144 2010-07-30 12:37:36Z jmr@macports.org $
 #
 # Copyright (c) 2002 - 2004 Apple Computer, Inc.
 # Copyright (c) 2004 - 2005 Paul Guyot <pguyot@kallisys.net>
@@ -188,6 +188,7 @@ proc portchecksum::checksum_start {args} {
 #
 proc portchecksum::checksum_main {args} {
     global UI_PREFIX all_dist_files checksum_types checksums_array portverbose checksum.skip
+    global usealtworkpath altprefix
 
     # If no files have been downloaded, there is nothing to checksum.
     if {![info exists all_dist_files]} {
@@ -217,6 +218,13 @@ proc portchecksum::checksum_main {args} {
 
             # get the full path of the distfile.
             set fullpath [file join $distpath $distfile]
+            if {![file isfile $fullpath]} {
+                if {!$usealtworkpath && [file isfile "${altprefix}${fullpath}"]} {
+                    set fullpath "${altprefix}${fullpath}"
+                } else {
+                    return -code error "$distfile does not exist in $distpath"
+                }
+            }
 
             # check that there is at least one checksum for the distfile.
             if {![info exists checksums_array($distfile)]} {
@@ -236,13 +244,19 @@ proc portchecksum::checksum_main {args} {
                         ui_debug "[format [msgcat::mc "Correct (%s) checksum for %s"] $type $distfile]"
                     } else {
                         ui_error "[format [msgcat::mc "Checksum (%s) mismatch for %s"] $type $distfile]"
-                        ui_info "[format [msgcat::mc "Portfile checksum: %s %s %s"] $distfile $type $sum]"
+                        ui_info_fetch "[format [msgcat::mc "Portfile checksum: %s %s %s"] $distfile $type $sum]"
                         ui_info "[format [msgcat::mc "Distfile checksum: %s %s %s"] $distfile $type $calculated_sum]"
 
                         # Raise the failure flag
                         set fail yes
                     }
                 }
+                if {[tbool fail] && ![regexp {\.html?$} ${distfile}] &&
+                    ![catch {strsed [exec [findBinary file $portutil::autoconf::file_path] $fullpath --brief --mime] {s/;.*$//}} mimetype]
+                    && "text/html" == $mimetype} {
+                    # file --mime-type would be preferable to file --mime and strsed, but is only available as of Snow Leopard
+                    set wrong_mimetype yes
+                }
             }
 
         }
@@ -261,6 +275,9 @@ proc portchecksum::checksum_main {args} {
             }
 
             set fullpath [file join $distpath $distfile]
+            if {![file isfile $fullpath] && (!$usealtworkpath && [file isfile "${altprefix}${fullpath}"])} {
+                set fullpath "${altprefix}${fullpath}"
+            }
             foreach type $checksum_types {
                 lappend sums [format "%-8s%s" $type [calc_$type $fullpath]]
             }
@@ -268,6 +285,18 @@ proc portchecksum::checksum_main {args} {
         ui_info "The correct checksum line may be:"
         ui_info [format "%-20s%s" "checksums" [join $sums [format " \\\n%-20s" ""]]]
 
+        if {[tbool wrong_mimetype]} {
+            # We got an HTML file, though the distfile name does not suggest that one was
+            # expected. Probably a helpful DNS server sent us to its search results page
+            # instead of admitting that the server we asked for doesn't exist, or a mirror that
+            # no longer has the file served its error page with a 200 response.
+            ui_msg "***"
+            ui_msg "The non-matching file appears to be HTML. See this page for possible reasons"
+            ui_msg "for the checksum mismatch:"
+            ui_msg "<http://trac.macports.org/wiki/MisbehavingServers>"
+            ui_msg "***"
+        }
+
         return -code error "[msgcat::mc "Unable to verify file checksums"]"
     }
 
