@@ -1,6 +1,6 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
 # portuninstall.tcl
-# $Id: portuninstall.tcl 68865 2010-06-16 14:26:25Z jmr@macports.org $
+# $Id: portuninstall.tcl 69319 2010-06-30 07:52:27Z jmr@macports.org $
 #
 # Copyright (c) 2002 - 2003 Apple Inc.
 # All rights reserved.
@@ -108,17 +108,17 @@ proc uninstall {portname {v ""} optionslist} {
     } elseif { [llength $ilist] == 1 } {
         if {$use_reg2} {
             set port [lindex $ilist 0]
-            if {$v == ""} {
-                set v "[$port version]_[$port revision][$port variants]"
-            }
+            set version [$port version]
+            set revision [$port revision]
+            set variants [$port variants]
         } else {
             set version [lindex [lindex $ilist 0] 1]
             set revision [lindex [lindex $ilist 0] 2]
             set variants [lindex [lindex $ilist 0] 3]
             set active [lindex [lindex $ilist 0] 4]
-            if {$v == ""} {
-                set v "${version}_${revision}${variants}"
-            }
+        }
+        if {$v == ""} {
+            set v "${version}_${revision}${variants}"
         }
     } else {
         throw registry::invalid "Registry error: $portname not registered as installed"
